package com.s3ai.corporate_app2.controllers

import com.fasterxml.jackson.databind.ObjectMapper
import com.s3ai.corporate_app2.UserService
import org.assertj.core.api.Assertions.assertThat
import org.hamcrest.Matchers
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers
import java.util.*


@SpringBootTest
@AutoConfigureMockMvc
class UsersTest {
    @Autowired
    private lateinit var service: UserService

    @Autowired
    private lateinit var controller: UsersController

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        assertThat(controller).isNotNull
    }

    @Test
    @Throws(Exception::class)
    @Sql("/usersFill.sql")
    fun updateUserTest() {
        val user = service.findAll().random()
        print(user.id)

        user.name = UUID.randomUUID().toString()

        mvc.perform(MockMvcRequestBuilders.post("/users/update")
                .contentType(MediaType.APPLICATION_JSON)
                .content(ObjectMapper().writeValueAsString(user)))
                .andExpect(MockMvcResultMatchers.status().isFound)
                .andExpect(MockMvcResultMatchers.redirectedUrl("/users/list"))

        val updatedUser = service.findById(user.id!!)
        assertThat(user == updatedUser)
    }

    @Test
    @Throws(Exception::class)
    fun editRedirectToCreateIfIdEmptyTest() {
        mvc.perform(MockMvcRequestBuilders.get("/users/edit"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andExpect(MockMvcResultMatchers.model().attribute("action", Matchers.equalTo("Create")))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/usersFill.sql")
    fun editPageTest() {
        val randomUserId = service.findAll().map { e -> e.id }.distinct().random()
        mvc.perform(MockMvcRequestBuilders.get("/users/edit?id=${randomUserId}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andExpect(MockMvcResultMatchers.model().attribute("action", Matchers.equalTo("Edit")))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql")
    fun listPageTest() {
        mvc.perform(MockMvcRequestBuilders.get("/users/list"))
                .andExpect(MockMvcResultMatchers.status().isOk)
    }
}
